@@ -190,14 +190,14 @@ class IpaddrUnitTest(unittest.TestCase):
 
     if ipaddr._compat_has_real_bytes: # on python3+
         def testIpFromPacked(self):
-            ip = ipaddr.IP
+            ip = ipaddr.IPNetwork
 
             self.assertEqual(self.ipv4.ip,
                              ip(_cb('\x01\x02\x03\x04')).ip)
             self.assertEqual(ip('255.254.253.252'),
                              ip(_cb('\xff\xfe\xfd\xfc')))
-            self.assertRaises(ValueError, ipaddr.IP, _cb('\x00' * 3))
-            self.assertRaises(ValueError, ipaddr.IP, _cb('\x00' * 5))
+            self.assertRaises(ValueError, ipaddr.IPNetwork, _cb('\x00' * 3))
+            self.assertRaises(ValueError, ipaddr.IPNetwork, _cb('\x00' * 5))
             self.assertEqual(self.ipv6.ip,
                              ip(_cb('\x20\x01\x06\x58\x02\x2a\xca\xfe'
                                '\x02\x00\x00\x00\x00\x00\x00\x01')).ip)
@@ -230,11 +230,11 @@ class IpaddrUnitTest(unittest.TestCase):
     def testZeroNetmask(self):
         ipv4_zero_netmask = ipaddr.IPv4Network('1.2.3.4/0')
         self.assertEqual(int(ipv4_zero_netmask.netmask), 0)
-        self.assert_(ipv4_zero_netmask._is_valid_netmask(str(0)))
+        self.assertTrue(ipv4_zero_netmask._is_valid_netmask(str(0)))
 
         ipv6_zero_netmask = ipaddr.IPv6Network('::1/0')
         self.assertEqual(int(ipv6_zero_netmask.netmask), 0)
-        self.assert_(ipv6_zero_netmask._is_valid_netmask(str(0)))
+        self.assertTrue(ipv6_zero_netmask._is_valid_netmask(str(0)))
 
     def testGetBroadcast(self):
         self.assertEqual(int(self.ipv4.broadcast), 16909311L)
@@ -416,16 +416,28 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertEqual('172.31.255.143', str(addr[-1]))
         self.assertEqual(addr_list[-1], addr[-1])
 
-    def testEquals(self):
+    def testEqual(self):
         self.assertTrue(self.ipv4 == ipaddr.IPv4Network('1.2.3.4/24'))
         self.assertFalse(self.ipv4 == ipaddr.IPv4Network('1.2.3.4/23'))
         self.assertFalse(self.ipv4 == ipaddr.IPv6Network('::1.2.3.4/24'))
         self.assertFalse(self.ipv4 == '')
         self.assertFalse(self.ipv4 == [])
         self.assertFalse(self.ipv4 == 2)
+        self.assertTrue(ipaddr.IPNetwork('1.1.1.1/32') ==
+                        ipaddr.IPAddress('1.1.1.1'))
+        self.assertTrue(ipaddr.IPNetwork('1.1.1.1/24') ==
+                        ipaddr.IPAddress('1.1.1.1'))
+        self.assertFalse(ipaddr.IPNetwork('1.1.1.0/24') ==
+                         ipaddr.IPAddress('1.1.1.1'))
 
         self.assertTrue(self.ipv6 ==
             ipaddr.IPv6Network('2001:658:22a:cafe:200::1/64'))
+        self.assertTrue(ipaddr.IPNetwork('::1/128') ==
+                        ipaddr.IPAddress('::1'))
+        self.assertTrue(ipaddr.IPNetwork('::1/127') ==
+                        ipaddr.IPAddress('::1'))
+        self.assertFalse(ipaddr.IPNetwork('::0/127') ==
+                         ipaddr.IPAddress('::1'))
         self.assertFalse(self.ipv6 ==
             ipaddr.IPv6Network('2001:658:22a:cafe:200::1/63'))
         self.assertFalse(self.ipv6 == ipaddr.IPv4Network('1.2.3.4/23'))
@@ -433,10 +445,8 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertFalse(self.ipv6 == [])
         self.assertFalse(self.ipv6 == 2)
 
-    def testNotEquals(self):
-        addr1 = ipaddr.IPAddress('1.2.3.4')
+    def testNotEqual(self):
         self.assertFalse(self.ipv4 != ipaddr.IPv4Network('1.2.3.4/24'))
-        self.assertFalse(self.ipv4 == addr1)
         self.assertTrue(self.ipv4 != ipaddr.IPv4Network('1.2.3.4/23'))
         self.assertTrue(self.ipv4 != ipaddr.IPv6Network('::1.2.3.4/24'))
         self.assertTrue(self.ipv4 != '')
@@ -446,7 +456,6 @@ class IpaddrUnitTest(unittest.TestCase):
         addr2 = ipaddr.IPAddress('2001:658:22a:cafe:200::1')
         self.assertFalse(self.ipv6 !=
             ipaddr.IPv6Network('2001:658:22a:cafe:200::1/64'))
-        self.assertFalse(self.ipv6 == addr2)
         self.assertTrue(self.ipv6 !=
             ipaddr.IPv6Network('2001:658:22a:cafe:200::1/63'))
         self.assertTrue(self.ipv6 != ipaddr.IPv4Network('1.2.3.4/23'))
@@ -455,15 +464,15 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertTrue(self.ipv6 != 2)
 
     def testSlash32Constructor(self):
-        self.assertEquals(str(ipaddr.IPv4Network('1.2.3.4/255.255.255.255')),
+        self.assertEqual(str(ipaddr.IPv4Network('1.2.3.4/255.255.255.255')),
                           '1.2.3.4/32')
 
     def testSlash128Constructor(self):
-        self.assertEquals(str(ipaddr.IPv6Network('::1/128')),
+        self.assertEqual(str(ipaddr.IPv6Network('::1/128')),
                                   '::1/128')
 
     def testSlash0Constructor(self):
-        self.assertEquals(str(ipaddr.IPv4Network('1.2.3.4/0.0.0.0')),
+        self.assertEqual(str(ipaddr.IPv4Network('1.2.3.4/0.0.0.0')),
                           '1.2.3.4/0')
 
     def testCollapsing(self):
@@ -581,9 +590,9 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertTrue(ip1 < ip3)
         self.assertTrue(ip3 > ip2)
 
-        self.assertEquals(ip1.compare_networks(ip2), 0)
+        self.assertEqual(ip1.compare_networks(ip2), 0)
         self.assertTrue(ip1._get_networks_key() == ip2._get_networks_key())
-        self.assertEquals(ip1.compare_networks(ip3), -1)
+        self.assertEqual(ip1.compare_networks(ip3), -1)
         self.assertTrue(ip1._get_networks_key() < ip3._get_networks_key())
 
         ip1 = ipaddr.IPv6Network('2001::2000/96')
@@ -592,9 +601,9 @@ class IpaddrUnitTest(unittest.TestCase):
 
         self.assertTrue(ip1 < ip3)
         self.assertTrue(ip3 > ip2)
-        self.assertEquals(ip1.compare_networks(ip2), 0)
+        self.assertEqual(ip1.compare_networks(ip2), 0)
         self.assertTrue(ip1._get_networks_key() == ip2._get_networks_key())
-        self.assertEquals(ip1.compare_networks(ip3), -1)
+        self.assertEqual(ip1.compare_networks(ip3), -1)
         self.assertTrue(ip1._get_networks_key() < ip3._get_networks_key())
 
         # Test comparing different protocols.
@@ -621,6 +630,21 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertFalse(ip2 > ip3)
         self.assertTrue(ip3 > ip2)
 
+        # Regression test for issue 28.
+        ip1 = ipaddr.IPNetwork('10.10.10.0/31')
+        ip2 = ipaddr.IPNetwork('10.10.10.0')
+        ip3 = ipaddr.IPNetwork('10.10.10.2/31')
+        ip4 = ipaddr.IPNetwork('10.10.10.2')
+        sorted = [ip1, ip2, ip3, ip4]
+        unsorted = [ip2, ip4, ip1, ip3]
+        unsorted.sort()
+        self.assertEqual(sorted, unsorted)
+        unsorted = [ip4, ip1, ip3, ip2]
+        unsorted.sort()
+        self.assertEqual(sorted, unsorted)
+        self.assertRaises(TypeError, ip1.__lt__, ipaddr.IPAddress('10.10.10.0'))
+        self.assertRaises(TypeError, ip2.__lt__, ipaddr.IPAddress('10.10.10.0'))
+
         # <=, >=
         self.assertTrue(ipaddr.IPNetwork('1.1.1.1') <=
                         ipaddr.IPNetwork('1.1.1.1'))
@@ -649,16 +673,29 @@ class IpaddrUnitTest(unittest.TestCase):
         ipv4_string = '192.168.0.1'
         ipv4 = ipaddr.IPv4Network(ipv4_string)
         v4compat_ipv6 = ipaddr.IPv6Network('::%s' % ipv4_string)
-        self.assertEquals(int(v4compat_ipv6.ip), int(ipv4.ip))
+        self.assertEqual(int(v4compat_ipv6.ip), int(ipv4.ip))
         v4mapped_ipv6 = ipaddr.IPv6Network('::ffff:%s' % ipv4_string)
-        self.assertNotEquals(v4mapped_ipv6.ip, ipv4.ip)
+        self.assertNotEqual(v4mapped_ipv6.ip, ipv4.ip)
         self.assertRaises(ipaddr.AddressValueError, ipaddr.IPv6Network,
                           '2001:1.1.1.1:1.1.1.1')
 
+    # Issue 67: IPv6 with embedded IPv4 address not recognized.
+    def testIPv6AddressTooLarge(self):
+        # RFC4291 2.5.5.2
+        self.assertEqual(ipaddr.IPAddress('::FFFF:192.0.2.1'),
+                          ipaddr.IPAddress('::FFFF:c000:201'))
+        # RFC4291 2.2 (part 3) x::d.d.d.d 
+        self.assertEqual(ipaddr.IPAddress('FFFF::192.0.2.1'),
+                          ipaddr.IPAddress('FFFF::c000:201'))
+
     def testIPVersion(self):
         self.assertEqual(self.ipv4.version, 4)
         self.assertEqual(self.ipv6.version, 6)
 
+    def testMaxPrefixLength(self):
+        self.assertEqual(self.ipv4.max_prefixlen, 32)
+        self.assertEqual(self.ipv6.max_prefixlen, 128)
+
     def testPacked(self):
         self.assertEqual(self.ipv4.packed,
                          _cb('\x01\x02\x03\x04'))
@@ -675,129 +712,129 @@ class IpaddrUnitTest(unittest.TestCase):
 
     def testIpStrFromPrefixlen(self):
         ipv4 = ipaddr.IPv4Network('1.2.3.4/24')
-        self.assertEquals(ipv4._ip_string_from_prefix(), '255.255.255.0')
-        self.assertEquals(ipv4._ip_string_from_prefix(28), '255.255.255.240')
+        self.assertEqual(ipv4._ip_string_from_prefix(), '255.255.255.0')
+        self.assertEqual(ipv4._ip_string_from_prefix(28), '255.255.255.240')
 
     def testIpType(self):
         ipv4net = ipaddr.IPNetwork('1.2.3.4')
         ipv4addr = ipaddr.IPAddress('1.2.3.4')
         ipv6net = ipaddr.IPNetwork('::1.2.3.4')
         ipv6addr = ipaddr.IPAddress('::1.2.3.4')
-        self.assertEquals(ipaddr.IPv4Network, type(ipv4net))
-        self.assertEquals(ipaddr.IPv4Address, type(ipv4addr))
-        self.assertEquals(ipaddr.IPv6Network, type(ipv6net))
-        self.assertEquals(ipaddr.IPv6Address, type(ipv6addr))
+        self.assertEqual(ipaddr.IPv4Network, type(ipv4net))
+        self.assertEqual(ipaddr.IPv4Address, type(ipv4addr))
+        self.assertEqual(ipaddr.IPv6Network, type(ipv6net))
+        self.assertEqual(ipaddr.IPv6Address, type(ipv6addr))
 
     def testReservedIpv4(self):
         # test networks
-        self.assertEquals(True, ipaddr.IPNetwork('224.1.1.1/31').is_multicast)
-        self.assertEquals(False, ipaddr.IPNetwork('240.0.0.0').is_multicast)
+        self.assertEqual(True, ipaddr.IPNetwork('224.1.1.1/31').is_multicast)
+        self.assertEqual(False, ipaddr.IPNetwork('240.0.0.0').is_multicast)
 
-        self.assertEquals(True, ipaddr.IPNetwork('192.168.1.1/17').is_private)
-        self.assertEquals(False, ipaddr.IPNetwork('192.169.0.0').is_private)
-        self.assertEquals(True, ipaddr.IPNetwork('10.255.255.255').is_private)
-        self.assertEquals(False, ipaddr.IPNetwork('11.0.0.0').is_private)
-        self.assertEquals(True, ipaddr.IPNetwork('172.31.255.255').is_private)
-        self.assertEquals(False, ipaddr.IPNetwork('172.32.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPNetwork('192.168.1.1/17').is_private)
+        self.assertEqual(False, ipaddr.IPNetwork('192.169.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPNetwork('10.255.255.255').is_private)
+        self.assertEqual(False, ipaddr.IPNetwork('11.0.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPNetwork('172.31.255.255').is_private)
+        self.assertEqual(False, ipaddr.IPNetwork('172.32.0.0').is_private)
 
-        self.assertEquals(True,
+        self.assertEqual(True,
                           ipaddr.IPNetwork('169.254.100.200/24').is_link_local)
-        self.assertEquals(False,
+        self.assertEqual(False,
                           ipaddr.IPNetwork('169.255.100.200/24').is_link_local)
 
-        self.assertEquals(True,
+        self.assertEqual(True,
                           ipaddr.IPNetwork('127.100.200.254/32').is_loopback)
-        self.assertEquals(True, ipaddr.IPNetwork('127.42.0.0/16').is_loopback)
-        self.assertEquals(False, ipaddr.IPNetwork('128.0.0.0').is_loopback)
+        self.assertEqual(True, ipaddr.IPNetwork('127.42.0.0/16').is_loopback)
+        self.assertEqual(False, ipaddr.IPNetwork('128.0.0.0').is_loopback)
 
         # test addresses
-        self.assertEquals(True, ipaddr.IPAddress('224.1.1.1').is_multicast)
-        self.assertEquals(False, ipaddr.IPAddress('240.0.0.0').is_multicast)
+        self.assertEqual(True, ipaddr.IPAddress('224.1.1.1').is_multicast)
+        self.assertEqual(False, ipaddr.IPAddress('240.0.0.0').is_multicast)
 
-        self.assertEquals(True, ipaddr.IPAddress('192.168.1.1').is_private)
-        self.assertEquals(False, ipaddr.IPAddress('192.169.0.0').is_private)
-        self.assertEquals(True, ipaddr.IPAddress('10.255.255.255').is_private)
-        self.assertEquals(False, ipaddr.IPAddress('11.0.0.0').is_private)
-        self.assertEquals(True, ipaddr.IPAddress('172.31.255.255').is_private)
-        self.assertEquals(False, ipaddr.IPAddress('172.32.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPAddress('192.168.1.1').is_private)
+        self.assertEqual(False, ipaddr.IPAddress('192.169.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPAddress('10.255.255.255').is_private)
+        self.assertEqual(False, ipaddr.IPAddress('11.0.0.0').is_private)
+        self.assertEqual(True, ipaddr.IPAddress('172.31.255.255').is_private)
+        self.assertEqual(False, ipaddr.IPAddress('172.32.0.0').is_private)
 
-        self.assertEquals(True,
+        self.assertEqual(True,
                           ipaddr.IPAddress('169.254.100.200').is_link_local)
-        self.assertEquals(False,
+        self.assertEqual(False,
                           ipaddr.IPAddress('169.255.100.200').is_link_local)
 
-        self.assertEquals(True,
+        self.assertEqual(True,
                           ipaddr.IPAddress('127.100.200.254').is_loopback)
-        self.assertEquals(True, ipaddr.IPAddress('127.42.0.0').is_loopback)
-        self.assertEquals(False, ipaddr.IPAddress('128.0.0.0').is_loopback)
-        self.assertEquals(True, ipaddr.IPNetwork('0.0.0.0').is_unspecified)
+        self.assertEqual(True, ipaddr.IPAddress('127.42.0.0').is_loopback)
+        self.assertEqual(False, ipaddr.IPAddress('128.0.0.0').is_loopback)
+        self.assertEqual(True, ipaddr.IPNetwork('0.0.0.0').is_unspecified)
 
     def testReservedIpv6(self):
 
-        self.assertEquals(True, ipaddr.IPNetwork('ffff::').is_multicast)
-        self.assertEquals(True, ipaddr.IPNetwork(2**128-1).is_multicast)
-        self.assertEquals(True, ipaddr.IPNetwork('ff00::').is_multicast)
-        self.assertEquals(False, ipaddr.IPNetwork('fdff::').is_multicast)
+        self.assertEqual(True, ipaddr.IPNetwork('ffff::').is_multicast)
+        self.assertEqual(True, ipaddr.IPNetwork(2**128-1).is_multicast)
+        self.assertEqual(True, ipaddr.IPNetwork('ff00::').is_multicast)
+        self.assertEqual(False, ipaddr.IPNetwork('fdff::').is_multicast)
 
-        self.assertEquals(True, ipaddr.IPNetwork('fecf::').is_site_local)
-        self.assertEquals(True, ipaddr.IPNetwork(
+        self.assertEqual(True, ipaddr.IPNetwork('fecf::').is_site_local)
+        self.assertEqual(True, ipaddr.IPNetwork(
                 'feff:ffff:ffff:ffff::').is_site_local)
-        self.assertEquals(False, ipaddr.IPNetwork('fbf:ffff::').is_site_local)
-        self.assertEquals(False, ipaddr.IPNetwork('ff00::').is_site_local)
+        self.assertEqual(False, ipaddr.IPNetwork('fbf:ffff::').is_site_local)
+        self.assertEqual(False, ipaddr.IPNetwork('ff00::').is_site_local)
 
-        self.assertEquals(True, ipaddr.IPNetwork('fc00::').is_private)
-        self.assertEquals(True, ipaddr.IPNetwork(
+        self.assertEqual(True, ipaddr.IPNetwork('fc00::').is_private)
+        self.assertEqual(True, ipaddr.IPNetwork(
                 'fc00:ffff:ffff:ffff::').is_private)
-        self.assertEquals(False, ipaddr.IPNetwork('fbff:ffff::').is_private)
-        self.assertEquals(False, ipaddr.IPNetwork('fe00::').is_private)
+        self.assertEqual(False, ipaddr.IPNetwork('fbff:ffff::').is_private)
+        self.assertEqual(False, ipaddr.IPNetwork('fe00::').is_private)
 
-        self.assertEquals(True, ipaddr.IPNetwork('fea0::').is_link_local)
-        self.assertEquals(True, ipaddr.IPNetwork('febf:ffff::').is_link_local)
-        self.assertEquals(False, ipaddr.IPNetwork('fe7f:ffff::').is_link_local)
-        self.assertEquals(False, ipaddr.IPNetwork('fec0::').is_link_local)
+        self.assertEqual(True, ipaddr.IPNetwork('fea0::').is_link_local)
+        self.assertEqual(True, ipaddr.IPNetwork('febf:ffff::').is_link_local)
+        self.assertEqual(False, ipaddr.IPNetwork('fe7f:ffff::').is_link_local)
+        self.assertEqual(False, ipaddr.IPNetwork('fec0::').is_link_local)
 
-        self.assertEquals(True, ipaddr.IPNetwork('0:0::0:01').is_loopback)
-        self.assertEquals(False, ipaddr.IPNetwork('::1/127').is_loopback)
-        self.assertEquals(False, ipaddr.IPNetwork('::').is_loopback)
-        self.assertEquals(False, ipaddr.IPNetwork('::2').is_loopback)
+        self.assertEqual(True, ipaddr.IPNetwork('0:0::0:01').is_loopback)
+        self.assertEqual(False, ipaddr.IPNetwork('::1/127').is_loopback)
+        self.assertEqual(False, ipaddr.IPNetwork('::').is_loopback)
+        self.assertEqual(False, ipaddr.IPNetwork('::2').is_loopback)
 
-        self.assertEquals(True, ipaddr.IPNetwork('0::0').is_unspecified)
-        self.assertEquals(False, ipaddr.IPNetwork('::1').is_unspecified)
-        self.assertEquals(False, ipaddr.IPNetwork('::/127').is_unspecified)
+        self.assertEqual(True, ipaddr.IPNetwork('0::0').is_unspecified)
+        self.assertEqual(False, ipaddr.IPNetwork('::1').is_unspecified)
+        self.assertEqual(False, ipaddr.IPNetwork('::/127').is_unspecified)
 
         # test addresses
-        self.assertEquals(True, ipaddr.IPAddress('ffff::').is_multicast)
-        self.assertEquals(True, ipaddr.IPAddress(2**128-1).is_multicast)
-        self.assertEquals(True, ipaddr.IPAddress('ff00::').is_multicast)
-        self.assertEquals(False, ipaddr.IPAddress('fdff::').is_multicast)
+        self.assertEqual(True, ipaddr.IPAddress('ffff::').is_multicast)
+        self.assertEqual(True, ipaddr.IPAddress(2**128-1).is_multicast)
+        self.assertEqual(True, ipaddr.IPAddress('ff00::').is_multicast)
+        self.assertEqual(False, ipaddr.IPAddress('fdff::').is_multicast)
 
-        self.assertEquals(True, ipaddr.IPAddress('fecf::').is_site_local)
-        self.assertEquals(True, ipaddr.IPAddress(
+        self.assertEqual(True, ipaddr.IPAddress('fecf::').is_site_local)
+        self.assertEqual(True, ipaddr.IPAddress(
                 'feff:ffff:ffff:ffff::').is_site_local)
-        self.assertEquals(False, ipaddr.IPAddress('fbf:ffff::').is_site_local)
-        self.assertEquals(False, ipaddr.IPAddress('ff00::').is_site_local)
+        self.assertEqual(False, ipaddr.IPAddress('fbf:ffff::').is_site_local)
+        self.assertEqual(False, ipaddr.IPAddress('ff00::').is_site_local)
 
-        self.assertEquals(True, ipaddr.IPAddress('fc00::').is_private)
-        self.assertEquals(True, ipaddr.IPAddress(
+        self.assertEqual(True, ipaddr.IPAddress('fc00::').is_private)
+        self.assertEqual(True, ipaddr.IPAddress(
                 'fc00:ffff:ffff:ffff::').is_private)
-        self.assertEquals(False, ipaddr.IPAddress('fbff:ffff::').is_private)
-        self.assertEquals(False, ipaddr.IPAddress('fe00::').is_private)
+        self.assertEqual(False, ipaddr.IPAddress('fbff:ffff::').is_private)
+        self.assertEqual(False, ipaddr.IPAddress('fe00::').is_private)
 
-        self.assertEquals(True, ipaddr.IPAddress('fea0::').is_link_local)
-        self.assertEquals(True, ipaddr.IPAddress('febf:ffff::').is_link_local)
-        self.assertEquals(False, ipaddr.IPAddress('fe7f:ffff::').is_link_local)
-        self.assertEquals(False, ipaddr.IPAddress('fec0::').is_link_local)
+        self.assertEqual(True, ipaddr.IPAddress('fea0::').is_link_local)
+        self.assertEqual(True, ipaddr.IPAddress('febf:ffff::').is_link_local)
+        self.assertEqual(False, ipaddr.IPAddress('fe7f:ffff::').is_link_local)
+        self.assertEqual(False, ipaddr.IPAddress('fec0::').is_link_local)
 
-        self.assertEquals(True, ipaddr.IPAddress('0:0::0:01').is_loopback)
-        self.assertEquals(True, ipaddr.IPAddress('::1').is_loopback)
-        self.assertEquals(False, ipaddr.IPAddress('::2').is_loopback)
+        self.assertEqual(True, ipaddr.IPAddress('0:0::0:01').is_loopback)
+        self.assertEqual(True, ipaddr.IPAddress('::1').is_loopback)
+        self.assertEqual(False, ipaddr.IPAddress('::2').is_loopback)
 
-        self.assertEquals(True, ipaddr.IPAddress('0::0').is_unspecified)
-        self.assertEquals(False, ipaddr.IPAddress('::1').is_unspecified)
+        self.assertEqual(True, ipaddr.IPAddress('0::0').is_unspecified)
+        self.assertEqual(False, ipaddr.IPAddress('::1').is_unspecified)
 
         # some generic IETF reserved addresses
-        self.assertEquals(True, ipaddr.IPAddress('100::').is_reserved)
-        self.assertEquals(True, ipaddr.IPNetwork('4000::1/128').is_reserved)
+        self.assertEqual(True, ipaddr.IPAddress('100::').is_reserved)
+        self.assertEqual(True, ipaddr.IPNetwork('4000::1/128').is_reserved)
 
     def testIpv4Mapped(self):
         self.assertEqual(ipaddr.IPAddress('::ffff:192.168.1.1').ipv4_mapped,
@@ -819,10 +856,14 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertEqual(addr1.address_exclude(addr1), [])
 
     def testHash(self):
-        self.assertEquals(hash(ipaddr.IPNetwork('10.1.1.0/24')),
+        self.assertEqual(hash(ipaddr.IPNetwork('10.1.1.0/24')),
                           hash(ipaddr.IPNetwork('10.1.1.0/24')))
-        self.assertEquals(hash(ipaddr.IPAddress('10.1.1.0')),
+        self.assertEqual(hash(ipaddr.IPAddress('10.1.1.0')),
                           hash(ipaddr.IPAddress('10.1.1.0')))
+        # i70
+        self.assertEqual(hash(ipaddr.IPAddress('1.2.3.4')),
+                          hash(ipaddr.IPAddress(
+                    long(ipaddr.IPAddress('1.2.3.4')._ip))))
         ip1 = ipaddr.IPAddress('10.1.1.0')
         ip2 = ipaddr.IPAddress('1::')
         dummy = {}
@@ -864,14 +905,18 @@ class IpaddrUnitTest(unittest.TestCase):
             '2001:658:22a:cafe::/66',
             }
         for uncompressed, compressed in test_addresses.items():
-            self.assertEquals(compressed, str(ipaddr.IPv6Network(uncompressed)))
+            self.assertEqual(compressed, str(ipaddr.IPv6Network(uncompressed)))
 
     def testExplodeShortHandIpStr(self):
         addr1 = ipaddr.IPv6Network('2001::1')
+        addr2 = ipaddr.IPv6Address('2001:0:5ef5:79fd:0:59d:a0e5:ba1')
         self.assertEqual('2001:0000:0000:0000:0000:0000:0000:0001',
                          addr1._explode_shorthand_ip_string(str(addr1.ip)))
         self.assertEqual('0000:0000:0000:0000:0000:0000:0000:0001',
                          ipaddr.IPv6Network('::1/128').exploded)
+        # issue 77
+        self.assertEqual('2001:0000:5ef5:79fd:0000:059d:a0e5:0ba1',
+                         addr2.exploded)
 
     def testIntRepresentation(self):
         self.assertEqual(16909060, int(self.ipv4))
@@ -934,7 +979,7 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertEqual(str(self.ipv6.with_netmask),
                          '2001:658:22a:cafe:200::1/64')
         # this probably don't make much sense, but it's included for
-        # compatability with ipv4
+        # compatibility with ipv4
         self.assertEqual(str(self.ipv6.with_hostmask),
                          '2001:658:22a:cafe:200::1/::ffff:ffff:ffff:ffff')
 
@@ -985,5 +1030,30 @@ class IpaddrUnitTest(unittest.TestCase):
         self.assertFalse(ip._is_valid_ip('2001:658:22a:zzzz:200::1'))
         self.assertFalse(ip._is_valid_ip('2001:658:22a:cafe1:200::1'))
 
+    def testTeredo(self):
+        # stolen from wikipedia
+        server = ipaddr.IPv4Address('65.54.227.120')
+        client = ipaddr.IPv4Address('192.0.2.45')
+        teredo_addr = '2001:0000:4136:e378:8000:63bf:3fff:fdd2'
+        self.assertEqual((server, client),
+                         ipaddr.IPAddress(teredo_addr).teredo)
+        bad_addr = '2000::4136:e378:8000:63bf:3fff:fdd2'
+        self.assertFalse(ipaddr.IPAddress(bad_addr).teredo)
+
+        # i77
+        teredo_addr = ipaddr.IPv6Address('2001:0:5ef5:79fd:0:59d:a0e5:ba1')
+        self.assertEqual((ipaddr.IPv4Address('94.245.121.253'),
+                          ipaddr.IPv4Address('95.26.244.94')),
+                         teredo_addr.teredo)
+
+
+    def testsixtofour(self):
+        sixtofouraddr = ipaddr.IPAddress('2002:ac1d:2d64::1')
+        bad_addr = ipaddr.IPAddress('2000:ac1d:2d64::1')
+        self.assertEqual(ipaddr.IPv4Address('172.29.45.100'),
+                         sixtofouraddr.sixtofour)
+        self.assertFalse(bad_addr.sixtofour)
+
+
 if __name__ == '__main__':
     unittest.main()
